@@ -31,8 +31,7 @@
 /**
  * GdkMonitor:
  *
- * `GdkMonitor` objects represent the individual outputs that are
- * associated with a `GdkDisplay`.
+ * Represents the individual outputs that are associated with a `GdkDisplay`.
  *
  * `GdkDisplay` keeps a `GListModel` to enumerate and monitor
  * monitors with [method@Gdk.Display.get_monitors]. You can use
@@ -187,7 +186,7 @@ gdk_monitor_class_init (GdkMonitorClass *class)
   object_class->set_property = gdk_monitor_set_property;
 
   /**
-   * GdkMonitor:description: (attributes org.gtk.Property.get=gdk_monitor_get_description)
+   * GdkMonitor:description:
    *
    * A short description of the monitor, meant for display to the user.
    *
@@ -199,7 +198,7 @@ gdk_monitor_class_init (GdkMonitorClass *class)
                          G_PARAM_READABLE | G_PARAM_STATIC_STRINGS);
 
   /**
-   * GdkMonitor:display: (attributes org.gtk.Property.get=gdk_monitor_get_display)
+   * GdkMonitor:display:
    *
    * The `GdkDisplay` of the monitor.
    */
@@ -209,7 +208,7 @@ gdk_monitor_class_init (GdkMonitorClass *class)
                          G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY | G_PARAM_STATIC_STRINGS);
 
   /**
-   * GdkMonitor:manufacturer: (attributes org.gtk.Property.get=gdk_monitor_get_manufacturer)
+   * GdkMonitor:manufacturer:
    *
    * The manufacturer name.
    */
@@ -219,7 +218,7 @@ gdk_monitor_class_init (GdkMonitorClass *class)
                          G_PARAM_READABLE | G_PARAM_STATIC_STRINGS);
 
   /**
-   * GdkMonitor:model: (attributes org.gtk.Property.get=gdk_monitor_get_model)
+   * GdkMonitor:model:
    *
    * The model name.
    */
@@ -229,7 +228,7 @@ gdk_monitor_class_init (GdkMonitorClass *class)
                          G_PARAM_READABLE | G_PARAM_STATIC_STRINGS);
 
   /**
-   * GdkMonitor:connector: (attributes org.gtk.Property.get=gdk_monitor_get_connector)
+   * GdkMonitor:connector:
    *
    * The connector name.
    */
@@ -239,7 +238,7 @@ gdk_monitor_class_init (GdkMonitorClass *class)
                          G_PARAM_READABLE | G_PARAM_STATIC_STRINGS);
 
   /**
-   * GdkMonitor:scale-factor: (attributes org.gtk.Property.get=gdk_monitor_get_scale_factor)
+   * GdkMonitor:scale-factor:
    *
    * The scale factor.
    *
@@ -253,7 +252,7 @@ gdk_monitor_class_init (GdkMonitorClass *class)
                       G_PARAM_READABLE | G_PARAM_STATIC_STRINGS);
 
   /**
-   * GdkMonitor:scale: (attributes org.gtk.Property.get=gdk_monitor_get_scale)
+   * GdkMonitor:scale:
    *
    * The scale of the monitor.
    *
@@ -265,7 +264,7 @@ gdk_monitor_class_init (GdkMonitorClass *class)
                         G_PARAM_READABLE | G_PARAM_STATIC_STRINGS);
 
   /**
-   * GdkMonitor:geometry: (attributes org.gtk.Property.get=gdk_monitor_get_geometry)
+   * GdkMonitor:geometry:
    *
    * The geometry of the monitor.
    */
@@ -275,7 +274,7 @@ gdk_monitor_class_init (GdkMonitorClass *class)
                         G_PARAM_READABLE | G_PARAM_STATIC_STRINGS);
 
   /**
-   * GdkMonitor:width-mm: (attributes org.gtk.Property.get=gdk_monitor_get_width_mm)
+   * GdkMonitor:width-mm:
    *
    * The width of the monitor, in millimeters.
    */
@@ -286,7 +285,7 @@ gdk_monitor_class_init (GdkMonitorClass *class)
                       G_PARAM_READABLE | G_PARAM_STATIC_STRINGS);
 
   /**
-   * GdkMonitor:height-mm: (attributes org.gtk.Property.get=gdk_monitor_get_height_mm)
+   * GdkMonitor:height-mm:
    *
    * The height of the monitor, in millimeters.
    */
@@ -297,7 +296,7 @@ gdk_monitor_class_init (GdkMonitorClass *class)
                       G_PARAM_READABLE | G_PARAM_STATIC_STRINGS);
 
   /**
-   * GdkMonitor:refresh-rate: (attributes org.gtk.Property.get=gdk_monitor_get_refresh_rate)
+   * GdkMonitor:refresh-rate:
    *
    * The refresh rate, in milli-Hertz.
    */
@@ -308,7 +307,7 @@ gdk_monitor_class_init (GdkMonitorClass *class)
                       G_PARAM_READABLE | G_PARAM_STATIC_STRINGS);
 
   /**
-   * GdkMonitor:subpixel-layout: (attributes org.gtk.Property.get=gdk_monitor_get_subpixel_layout)
+   * GdkMonitor:subpixel-layout:
    *
    * The subpixel layout.
    */
@@ -319,7 +318,7 @@ gdk_monitor_class_init (GdkMonitorClass *class)
                        G_PARAM_READABLE | G_PARAM_STATIC_STRINGS);
 
   /**
-   * GdkMonitor:valid: (attributes org.gtk.Property.get=gdk_monitor_is_valid)
+   * GdkMonitor:valid: (getter is_valid)
    *
    * Whether the object is still valid.
    */
@@ -346,7 +345,7 @@ gdk_monitor_class_init (GdkMonitorClass *class)
 }
 
 /**
- * gdk_monitor_get_display: (attributes org.gtk.Method.get_property=display)
+ * gdk_monitor_get_display:
  * @monitor: a `GdkMonitor`
  *
  * Gets the display that this monitor belongs to.
@@ -362,7 +361,7 @@ gdk_monitor_get_display (GdkMonitor *monitor)
 }
 
 /**
- * gdk_monitor_get_geometry: (attributes org.gtk.Method.get_property=geometry)
+ * gdk_monitor_get_geometry:
  * @monitor: a `GdkMonitor`
  * @geometry: (out): a `GdkRectangle` to be filled with the monitor geometry
  *
@@ -383,7 +382,7 @@ gdk_monitor_get_geometry (GdkMonitor   *monitor,
 }
 
 /**
- * gdk_monitor_get_width_mm: (attributes org.gtk.Method.get_property=width-mm)
+ * gdk_monitor_get_width_mm:
  * @monitor: a `GdkMonitor`
  *
  * Gets the width in millimeters of the monitor.
@@ -399,7 +398,7 @@ gdk_monitor_get_width_mm (GdkMonitor *monitor)
 }
 
 /**
- * gdk_monitor_get_height_mm: (attributes org.gtk.Method.get_property=height-mm)
+ * gdk_monitor_get_height_mm:
  * @monitor: a `GdkMonitor`
  *
  * Gets the height in millimeters of the monitor.
@@ -415,7 +414,7 @@ gdk_monitor_get_height_mm (GdkMonitor *monitor)
 }
 
 /**
- * gdk_monitor_get_connector: (attributes org.gtk.Method.get_property=connector)
+ * gdk_monitor_get_connector:
  * @monitor: a `GdkMonitor`
  *
  * Gets the name of the monitor's connector, if available.
@@ -435,7 +434,7 @@ gdk_monitor_get_connector (GdkMonitor *monitor)
 }
 
 /**
- * gdk_monitor_get_manufacturer: (attributes org.gtk.Method.get_property=manufacturer)
+ * gdk_monitor_get_manufacturer:
  * @monitor: a `GdkMonitor`
  *
  * Gets the name or PNP ID of the monitor's manufacturer.
@@ -457,7 +456,7 @@ gdk_monitor_get_manufacturer (GdkMonitor *monitor)
 }
 
 /**
- * gdk_monitor_get_model: (attributes org.gtk.Method.get_property=model)
+ * gdk_monitor_get_model:
  * @monitor: a `GdkMonitor`
  *
  * Gets the string identifying the monitor model, if available.
@@ -473,7 +472,7 @@ gdk_monitor_get_model (GdkMonitor *monitor)
 }
 
 /**
- * gdk_monitor_get_scale_factor: (attributes org.gtk.Method.get_property=scale-factor)
+ * gdk_monitor_get_scale_factor:
  * @monitor: a `GdkMonitor`
  *
  * Gets the internal scale factor that maps from monitor coordinates
@@ -497,7 +496,7 @@ gdk_monitor_get_scale_factor (GdkMonitor *monitor)
 }
 
 /**
- * gdk_monitor_get_scale: (attributes org.gtk.Method.get_property=scale)
+ * gdk_monitor_get_scale:
  * @monitor: a `GdkMonitor`
  *
  * Gets the internal scale factor that maps from monitor coordinates
@@ -520,7 +519,7 @@ gdk_monitor_get_scale (GdkMonitor *monitor)
 }
 
 /**
- * gdk_monitor_get_refresh_rate: (attributes org.gtk.Method.get_property=refresh-rate)
+ * gdk_monitor_get_refresh_rate:
  * @monitor: a `GdkMonitor`
  *
  * Gets the refresh rate of the monitor, if available.
@@ -539,7 +538,7 @@ gdk_monitor_get_refresh_rate (GdkMonitor *monitor)
 }
 
 /**
- * gdk_monitor_get_subpixel_layout: (attributes org.gtk.Method.get_property=subpixel-layout)
+ * gdk_monitor_get_subpixel_layout:
  * @monitor: a `GdkMonitor`
  *
  * Gets information about the layout of red, green and blue
@@ -649,7 +648,7 @@ void
 gdk_monitor_set_scale (GdkMonitor *monitor,
                        double      scale)
 {
-  g_return_if_fail (scale >= 1.);
+  g_return_if_fail (scale > 0.);
 
   monitor->scale_set = TRUE;
 
@@ -696,7 +695,7 @@ gdk_monitor_invalidate (GdkMonitor *monitor)
 }
 
 /**
- * gdk_monitor_is_valid: (attributes org.gtk.Method.get_property=valid)
+ * gdk_monitor_is_valid: (get-property valid)
  * @monitor: a `GdkMonitor`
  *
  * Returns %TRUE if the @monitor object corresponds to a
@@ -716,7 +715,7 @@ gdk_monitor_is_valid (GdkMonitor *monitor)
 }
 
 /**
- * gdk_monitor_get_description: (attributes org.gtk.Method.get_property=description)
+ * gdk_monitor_get_description:
  * @monitor: a `GdkMonitor`
  *
  * Gets a string describing the monitor, if available.
@@ -744,3 +743,11 @@ gdk_monitor_set_description (GdkMonitor *monitor,
   g_object_notify_by_pspec (G_OBJECT (monitor), props[PROP_DESCRIPTION]);
 }
 
+#define MM_PER_INCH 25.4
+
+double
+gdk_monitor_get_dpi (GdkMonitor *monitor)
+{
+  return MAX ((monitor->geometry.width * monitor->scale) / (monitor->width_mm / MM_PER_INCH),
+              (monitor->geometry.height * monitor->scale) / (monitor->height_mm / MM_PER_INCH));
+}
