@@ -30,7 +30,7 @@
 #include "gdkmacossurface.h"
 #include "edgesnapping.h"
 
-#define GDK_IS_MACOS_WINDOW(obj) ([obj isKindOfClass:[GdkMacosWindow class]])
+#define GDK_IS_MACOS_WINDOW(obj) (obj != nil && [obj isKindOfClass:[GdkMacosWindow class]])
 
 @interface GdkMacosWindow : NSWindow <NSDraggingSource, NSDraggingDestination> {
   GdkMacosSurface *gdk_surface;
@@ -52,6 +52,8 @@
   BOOL             inFullscreenTransition;
 }
 
+@property (nonatomic) BOOL showStandardWindowButtons;
+
 +(void)setContentViewClass:(Class)newViewClass;
 
 -(void)beginManualMove;
