@@ -17,8 +17,9 @@
  * SPDX-License-Identifier: LGPL-2.1-or-later
  */
 
-#ifndef __GDK_MACOS_H__
-#define __GDK_MACOS_H__
+#pragma once
+
+#define __GDKMACOS_H_INSIDE__
 
 #include <gdk/gdk.h>
 
@@ -29,4 +30,4 @@
 #include "gdkmacosmonitor.h"
 #include "gdkmacossurface.h"
 
-#endif /* __GDK_MACOS_H__ */
+#undef __GDKMACOS_H_INSIDE__
