@@ -201,26 +201,13 @@ get_egl_window_size (GdkSurface *surface,
   GdkDisplay *display = gdk_surface_get_display (surface);
   GdkWaylandSurface *impl = GDK_WAYLAND_SURFACE (surface);
 
-  if (GDK_DISPLAY_DEBUG_CHECK (display, GL_NO_FRACTIONAL))
-    {
-      *width = surface->width * gdk_fractional_scale_to_int (&impl->scale);
-      *height = surface->height * gdk_fractional_scale_to_int (&impl->scale);
+  *width = gdk_fractional_scale_scale (&impl->scale, surface->width),
+  *height = gdk_fractional_scale_scale (&impl->scale, surface->height);
 
-      GDK_DISPLAY_DEBUG (display, OPENGL, "Using integer scale %d for EGL window (%d %d => %d %d)",
-                         gdk_fractional_scale_to_int (&impl->scale),
-                         surface->width, surface->height,
-                         *width, *height);
-    }
-  else
-    {
-      *width = gdk_fractional_scale_scale (&impl->scale, surface->width),
-      *height = gdk_fractional_scale_scale (&impl->scale, surface->height);
-
-      GDK_DISPLAY_DEBUG (display, OPENGL, "Using fractional scale %g for EGL window (%d %d => %d %d)",
-                         gdk_fractional_scale_to_double (&impl->scale),
-                         surface->width, surface->height,
-                         *width, *height);
-    }
+  GDK_DISPLAY_DEBUG (display, OPENGL, "Using fractional scale %g for EGL window (%d %d => %d %d)",
+                     gdk_fractional_scale_to_double (&impl->scale),
+                     surface->width, surface->height,
+                     *width, *height);
 }
 
 void
@@ -449,6 +436,14 @@ gdk_wayland_surface_commit (GdkSurface *surface)
   wl_surface_commit (impl->display_server.wl_surface);
 }
 
+/**
+ * gdk_wayland_surface_force_next_commit:
+ * @surface: (type GdkWaylandSurface): a `GdkSurface`
+ *
+ * Forces next commit.
+ *
+ * Since: 4.18
+ */
 void
 gdk_wayland_surface_force_next_commit (GdkSurface *surface)
 {
