@@ -18,18 +18,21 @@
 #include "gdk/gdktextureprivate.h"
 #include "gdk/gdktexturedownloaderprivate.h"
 #include "gdk/gdkdrawcontextprivate.h"
+#include "gdk/gdkcolorstateprivate.h"
 
 #include <graphene.h>
 
 #define GSK_GPU_MAX_FRAMES 4
 
 static const GdkDebugKey gsk_gpu_optimization_keys[] = {
-  { "uber", GSK_GPU_OPTIMIZE_UBER, "Don't use the uber shader" },
-  { "clear", GSK_GPU_OPTIMIZE_CLEAR, "Use shaders instead of vkCmdClearAttachment()/glClear()" },
-  { "merge", GSK_GPU_OPTIMIZE_MERGE, "Use one vkCmdDraw()/glDrawArrays() per operation" },
-  { "blit", GSK_GPU_OPTIMIZE_BLIT, "Use shaders instead of vkCmdBlit()/glBlitFramebuffer()" },
-  { "gradients", GSK_GPU_OPTIMIZE_GRADIENTS, "Don't supersample gradients" },
-  { "mipmap", GSK_GPU_OPTIMIZE_MIPMAP, "Avoid creating mipmaps" },
+  { "clear",     GSK_GPU_OPTIMIZE_CLEAR,             "Use shaders instead of vkCmdClearAttachment()/glClear()" },
+  { "merge",     GSK_GPU_OPTIMIZE_MERGE,             "Use one vkCmdDraw()/glDrawArrays() per operation" },
+  { "blit",      GSK_GPU_OPTIMIZE_BLIT,              "Use shaders instead of vkCmdBlit()/glBlitFramebuffer()" },
+  { "gradients", GSK_GPU_OPTIMIZE_GRADIENTS,         "Don't supersample gradients" },
+  { "mipmap",    GSK_GPU_OPTIMIZE_MIPMAP,            "Avoid creating mipmaps" },
+  { "to-image",  GSK_GPU_OPTIMIZE_TO_IMAGE,          "Don't fast-path creation of images for nodes" },
+  { "occlusion", GSK_GPU_OPTIMIZE_OCCLUSION_CULLING, "Disable occlusion culling via opaque node tracking" },
+  { "repeat",    GSK_GPU_OPTIMIZE_REPEAT,            "Repeat drawing operations instead of using offscreen and GL_REPEAT" },
 };
 
 typedef struct _GskGpuRendererPrivate GskGpuRendererPrivate;
@@ -56,6 +59,19 @@ gsk_gpu_renderer_make_current (GskGpuRenderer *self)
   GSK_GPU_RENDERER_GET_CLASS (self)->make_current (self);
 }
 
+static gpointer
+gsk_gpu_renderer_save_current (GskGpuRenderer *self)
+{
+  return GSK_GPU_RENDERER_GET_CLASS (self)->save_current (self);
+}
+
+static void
+gsk_gpu_renderer_restore_current (GskGpuRenderer *self,
+                                  gpointer        current)
+{
+  GSK_GPU_RENDERER_GET_CLASS (self)->restore_current (self, current);
+}
+
 static GskGpuFrame *
 gsk_gpu_renderer_create_frame (GskGpuRenderer *self)
 {
@@ -70,96 +86,6 @@ gsk_gpu_renderer_create_frame (GskGpuRenderer *self)
   return result;
 }
 
-static void
-gsk_gpu_renderer_dmabuf_downloader_close (GdkDmabufDownloader *downloader)
-{
-  gsk_renderer_unrealize (GSK_RENDERER (downloader));
-}
-
-static gboolean
-gsk_gpu_renderer_dmabuf_downloader_supports (GdkDmabufDownloader  *downloader,
-                                             GdkDmabufTexture     *texture,
-                                             GError              **error)
-{
-  GskGpuRenderer *self = GSK_GPU_RENDERER (downloader);
-  const GdkDmabuf *dmabuf;
-  GdkDmabufFormats *formats;
-
-  dmabuf = gdk_dmabuf_texture_get_dmabuf (texture);
-
-  formats = GSK_GPU_RENDERER_GET_CLASS (self)->get_dmabuf_formats (self);
-
-  if (!gdk_dmabuf_formats_contains (formats, dmabuf->fourcc, dmabuf->modifier))
-    {
-      g_set_error (error,
-                   GDK_DMABUF_ERROR, GDK_DMABUF_ERROR_UNSUPPORTED_FORMAT,
-                   "Unsupported dmabuf format: %.4s:%#" G_GINT64_MODIFIER "x",
-                   (char *) &dmabuf->fourcc, dmabuf->modifier);
-      return FALSE;
-    }
-
-  return TRUE;
-}
-
-static void
-gsk_gpu_renderer_dmabuf_downloader_download (GdkDmabufDownloader *downloader,
-                                             GdkDmabufTexture    *texture,
-                                             GdkMemoryFormat      format,
-                                             guchar              *data,
-                                             gsize                stride)
-{
-  GskGpuRenderer *self = GSK_GPU_RENDERER (downloader);
-  GskGpuFrame *frame;
-
-  gsk_gpu_renderer_make_current (self);
-
-  frame = gsk_gpu_renderer_create_frame (self);
-
-  gsk_gpu_frame_download_texture (frame,
-                                  g_get_monotonic_time (),
-                                  GDK_TEXTURE (texture),
-                                  format,
-                                  data,
-                                  stride);
-
-  g_object_unref (frame);
-}
-
-static void
-gsk_gpu_renderer_dmabuf_downloader_init (GdkDmabufDownloaderInterface *iface)
-{
-  iface->close = gsk_gpu_renderer_dmabuf_downloader_close;
-  iface->supports = gsk_gpu_renderer_dmabuf_downloader_supports;
-  iface->download = gsk_gpu_renderer_dmabuf_downloader_download;
-}
-
-static cairo_region_t *
-get_render_region (GskGpuRenderer *self)
-{
-  GskGpuRendererPrivate *priv = gsk_gpu_renderer_get_instance_private (self);
-  const cairo_region_t *damage;
-  cairo_region_t *scaled_damage;
-  double scale;
-
-  scale = gsk_gpu_renderer_get_scale (self);
-
-  damage = gdk_draw_context_get_frame_region (priv->context);
-  scaled_damage = cairo_region_create ();
-  for (int i = 0; i < cairo_region_num_rectangles (damage); i++)
-    {
-      cairo_rectangle_int_t rect;
-      cairo_region_get_rectangle (damage, i, &rect);
-      cairo_region_union_rectangle (scaled_damage, &(cairo_rectangle_int_t) {
-                                      .x = (int) floor (rect.x * scale),
-                                      .y = (int) floor (rect.y * scale),
-                                      .width = (int) ceil ((rect.x + rect.width) * scale) - floor (rect.x * scale),
-                                      .height = (int) ceil ((rect.y + rect.height) * scale) - floor (rect.y * scale),
-                                    });
-    }
-
-  return scaled_damage;
-}
-
 static GskGpuFrame *
 gsk_gpu_renderer_get_frame (GskGpuRenderer *self)
 {
@@ -196,6 +122,91 @@ gsk_gpu_renderer_get_frame (GskGpuRenderer *self)
   return earliest_frame;
 }
 
+static void
+gsk_gpu_renderer_dmabuf_downloader_close (GdkDmabufDownloader *downloader)
+{
+  gsk_renderer_unrealize (GSK_RENDERER (downloader));
+}
+
+static gboolean
+gsk_gpu_renderer_dmabuf_downloader_download (GdkDmabufDownloader *downloader,
+                                             GdkDmabufTexture    *texture,
+                                             GdkMemoryFormat      format,
+                                             GdkColorState       *color_state,
+                                             guchar              *data,
+                                             gsize                stride)
+{
+  GskGpuRenderer *self = GSK_GPU_RENDERER (downloader);
+  GskGpuFrame *frame;
+  gpointer previous;
+  gboolean retval = FALSE;
+
+  previous = gsk_gpu_renderer_save_current (self);
+
+  gsk_gpu_renderer_make_current (self);
+
+  frame = gsk_gpu_renderer_get_frame (self);
+
+  if (gsk_gpu_frame_download_texture (frame,
+                                      g_get_monotonic_time (),
+                                      GDK_TEXTURE (texture),
+                                      format,
+                                      color_state,
+                                      data,
+                                      stride))
+    {
+      retval = TRUE;
+
+      GDK_DISPLAY_DEBUG (gdk_dmabuf_texture_get_display (texture), DMABUF,
+                         "Used %s for downloading %dx%d dmabuf (format %.4s:%#" G_GINT64_MODIFIER "x)",
+                         G_OBJECT_TYPE_NAME (downloader),
+                         gdk_texture_get_width (GDK_TEXTURE (texture)),
+                         gdk_texture_get_height (GDK_TEXTURE (texture)),
+                         (char *)&(gdk_dmabuf_texture_get_dmabuf (texture)->fourcc),
+                         gdk_dmabuf_texture_get_dmabuf (texture)->modifier);
+
+      gsk_gpu_frame_wait (frame);
+    }
+
+  gsk_gpu_renderer_restore_current (self, previous);
+
+  return retval;
+}
+
+static void
+gsk_gpu_renderer_dmabuf_downloader_init (GdkDmabufDownloaderInterface *iface)
+{
+  iface->close = gsk_gpu_renderer_dmabuf_downloader_close;
+  iface->download = gsk_gpu_renderer_dmabuf_downloader_download;
+}
+
+static cairo_region_t *
+get_render_region (GskGpuRenderer *self)
+{
+  GskGpuRendererPrivate *priv = gsk_gpu_renderer_get_instance_private (self);
+  const cairo_region_t *damage;
+  cairo_region_t *scaled_damage;
+  double scale;
+
+  scale = gdk_surface_get_scale (gdk_draw_context_get_surface (priv->context));
+
+  damage = gdk_draw_context_get_frame_region (priv->context);
+  scaled_damage = cairo_region_create ();
+  for (int i = 0; i < cairo_region_num_rectangles (damage); i++)
+    {
+      cairo_rectangle_int_t rect;
+      cairo_region_get_rectangle (damage, i, &rect);
+      cairo_region_union_rectangle (scaled_damage, &(cairo_rectangle_int_t) {
+                                      .x = (int) floor (rect.x * scale),
+                                      .y = (int) floor (rect.y * scale),
+                                      .width = (int) ceil ((rect.x + rect.width) * scale) - floor (rect.x * scale),
+                                      .height = (int) ceil ((rect.y + rect.height) * scale) - floor (rect.y * scale),
+                                    });
+    }
+
+  return scaled_damage;
+}
+
 static gboolean
 gsk_gpu_renderer_realize (GskRenderer  *renderer,
                           GdkDisplay   *display,
@@ -205,6 +216,7 @@ gsk_gpu_renderer_realize (GskRenderer  *renderer,
   GskGpuRenderer *self = GSK_GPU_RENDERER (renderer);
   GskGpuRendererPrivate *priv = gsk_gpu_renderer_get_instance_private (self);
   GskGpuOptimizations context_optimizations;
+  G_GNUC_UNUSED gint64 start_time = GDK_PROFILER_CURRENT_TIME;
 
   priv->device = GSK_GPU_RENDERER_GET_CLASS (self)->get_device (display, error);
   if (priv->device == NULL)
@@ -219,6 +231,8 @@ gsk_gpu_renderer_realize (GskRenderer  *renderer,
 
   priv->optimizations &= context_optimizations;
 
+  gdk_profiler_end_mark (start_time, "Realize GskGpuRenderer", NULL);
+
   return TRUE;
 }
 
@@ -255,10 +269,12 @@ gsk_gpu_renderer_fallback_render_texture (GskGpuRenderer        *self,
   gsize x, y, size, bpp, stride;
   GdkMemoryFormat format;
   GdkMemoryDepth depth;
+  GdkColorState *color_state;
   GBytes *bytes;
   guchar *data;
   GdkTexture *texture;
   GdkTextureDownloader downloader;
+  cairo_region_t *clip_region;
   GskGpuFrame *frame;
 
   max_size = gsk_gpu_device_get_max_image_size (priv->device);
@@ -294,18 +310,29 @@ gsk_gpu_renderer_fallback_render_texture (GskGpuRenderer        *self,
                                                           MIN (image_width, width - x),
                                                           MIN (image_height, height - y));
 
-          frame = gsk_gpu_renderer_create_frame (self);
+          if (gsk_gpu_image_get_flags (image) & GSK_GPU_IMAGE_SRGB)
+            color_state = GDK_COLOR_STATE_SRGB_LINEAR;
+          else
+            color_state = GDK_COLOR_STATE_SRGB;
+
+          clip_region = cairo_region_create_rectangle (&(cairo_rectangle_int_t) {
+                                                           0, 0,
+                                                           gsk_gpu_image_get_width (image),
+                                                           gsk_gpu_image_get_height (image)
+                                                       });
+          frame = gsk_gpu_renderer_get_frame (self);
           gsk_gpu_frame_render (frame,
                                 g_get_monotonic_time (),
                                 image,
-                                NULL,
+                                color_state,
+                                clip_region,
                                 root,
                                 &GRAPHENE_RECT_INIT (rounded_viewport->origin.x + x,
                                                      rounded_viewport->origin.y + y,
                                                      image_width,
                                                      image_height),
                                 &texture);
-          g_object_unref (frame);
+          gsk_gpu_frame_wait (frame);
 
           g_assert (texture);
           gdk_texture_downloader_init (&downloader, texture);
@@ -317,6 +344,9 @@ gsk_gpu_renderer_fallback_render_texture (GskGpuRenderer        *self,
           gdk_texture_downloader_finish (&downloader);
           g_object_unref (texture);
           g_clear_object (&image);
+
+          /* Let's GC like a madman, we draw oversized stuff and don't want to OOM */
+          gsk_gpu_device_maybe_gc (priv->device);
         }
     }
 
@@ -337,6 +367,8 @@ gsk_gpu_renderer_render_texture (GskRenderer           *renderer,
   GskGpuImage *image;
   GdkTexture *texture;
   graphene_rect_t rounded_viewport;
+  GdkColorState *color_state;
+  cairo_region_t *clip_region;
 
   gsk_gpu_device_maybe_gc (priv->device);
 
@@ -354,18 +386,30 @@ gsk_gpu_renderer_render_texture (GskRenderer           *renderer,
   if (image == NULL)
     return gsk_gpu_renderer_fallback_render_texture (self, root, &rounded_viewport);
 
-  frame = gsk_gpu_renderer_create_frame (self);
+  if (gsk_gpu_image_get_flags (image) & GSK_GPU_IMAGE_SRGB)
+    color_state = GDK_COLOR_STATE_SRGB_LINEAR;
+  else
+    color_state = GDK_COLOR_STATE_SRGB;
+
+  frame = gsk_gpu_renderer_get_frame (self);
+
+  clip_region = cairo_region_create_rectangle (&(cairo_rectangle_int_t) {
+                                                   0, 0,
+                                                   gsk_gpu_image_get_width (image),
+                                                   gsk_gpu_image_get_height (image)
+                                               });
 
   texture = NULL;
   gsk_gpu_frame_render (frame,
                         g_get_monotonic_time (),
                         image,
-                        NULL,
+                        color_state,
+                        clip_region,
                         root,
                         &rounded_viewport,
                         &texture);
 
-  g_object_unref (frame);
+  gsk_gpu_frame_wait (frame);
   g_object_unref (image);
 
   gsk_gpu_device_queue_gc (priv->device);
@@ -386,7 +430,10 @@ gsk_gpu_renderer_render (GskRenderer          *renderer,
   GskGpuFrame *frame;
   GskGpuImage *backbuffer;
   cairo_region_t *render_region;
+  graphene_rect_t opaque_tmp;
+  const graphene_rect_t *opaque;
   double scale;
+  GdkMemoryDepth depth;
 
   if (cairo_region_is_empty (region))
     {
@@ -394,23 +441,28 @@ gsk_gpu_renderer_render (GskRenderer          *renderer,
       return;
     }
 
-  gdk_draw_context_begin_frame_full (priv->context,
-                                     gsk_render_node_get_preferred_depth (root),
-                                     region);
-
   gsk_gpu_device_maybe_gc (priv->device);
 
   gsk_gpu_renderer_make_current (self);
 
+  depth = gsk_render_node_get_preferred_depth (root);
+  frame = gsk_gpu_renderer_get_frame (self);
+  scale = gdk_surface_get_scale (gdk_draw_context_get_surface (priv->context));
+
+  if (gsk_render_node_get_opaque_rect (root, &opaque_tmp))
+    opaque = &opaque_tmp;
+  else
+    opaque = NULL;
+  gsk_gpu_frame_begin (frame, priv->context, depth, region, opaque);
+
   backbuffer = GSK_GPU_RENDERER_GET_CLASS (self)->get_backbuffer (self);
 
-  frame = gsk_gpu_renderer_get_frame (self);
   render_region = get_render_region (self);
-  scale = gsk_gpu_renderer_get_scale (self);
 
   gsk_gpu_frame_render (frame,
                         g_get_monotonic_time (),
                         backbuffer,
+                        gdk_draw_context_get_color_state (priv->context),
                         render_region,
                         root,
                         &GRAPHENE_RECT_INIT (
@@ -420,22 +472,9 @@ gsk_gpu_renderer_render (GskRenderer          *renderer,
                         ),
                         NULL);
 
+  gsk_gpu_frame_end (frame, priv->context);
+
   gsk_gpu_device_queue_gc (priv->device);
-
-  gdk_draw_context_end_frame (priv->context);
-
-  g_clear_pointer (&render_region, cairo_region_destroy);
-}
-
-static double
-gsk_gpu_renderer_real_get_scale (GskGpuRenderer *self)
-{
-  GskGpuRendererPrivate *priv = gsk_gpu_renderer_get_instance_private (self);
-  GdkSurface *surface;
-
-  surface = gdk_draw_context_get_surface (priv->context);
-
-  return gdk_surface_get_scale (surface);
 }
 
 static void
@@ -454,9 +493,10 @@ gsk_gpu_renderer_class_init (GskGpuRendererClass *klass)
 
   klass->optimizations = -1;
   klass->optimizations &= ~gdk_parse_debug_var ("GSK_GPU_DISABLE",
-                                                gsk_gpu_optimization_keys,
-                                                G_N_ELEMENTS (gsk_gpu_optimization_keys));
-  klass->get_scale = gsk_gpu_renderer_real_get_scale;
+      "GSK_GPU_DISABLE can be set to of values which cause GSK to disable\n"
+      "certain optimizations in the \'ngl\' and \'vulkan\' renderers.\n",
+      gsk_gpu_optimization_keys,
+      G_N_ELEMENTS (gsk_gpu_optimization_keys));
 }
 
 static void
@@ -482,9 +522,3 @@ gsk_gpu_renderer_get_device (GskGpuRenderer *self)
 
   return priv->device;
 }
-
-double
-gsk_gpu_renderer_get_scale (GskGpuRenderer *self)
-{
-  return GSK_GPU_RENDERER_GET_CLASS (self)->get_scale (self);
-}
