@@ -42,16 +42,24 @@
 #include "gskrendererprivate.h"
 #include "gskrendernodeparserprivate.h"
 
+#include "gdk/gdkcairoprivate.h"
+#include "gdk/gdkcolorstateprivate.h"
+
 #include <graphene-gobject.h>
 
 #include <math.h>
 
 #include <gobject/gvaluecollector.h>
 
+/**
+ * gsk_serialization_error_quark:
+ *
+ * Registers an error quark for [class@Gsk.RenderNode] errors.
+ *
+ * Returns: the error quark
+ **/
 G_DEFINE_QUARK (gsk-serialization-error-quark, gsk_serialization_error)
 
-#define GSK_RENDER_NODE_GET_CLASS(obj)  (G_TYPE_INSTANCE_GET_CLASS ((obj), GSK_TYPE_RENDER_NODE, GskRenderNodeClass))
-
 
 static void
 value_render_node_init (GValue *value)
@@ -138,12 +146,6 @@ gsk_render_node_finalize (GskRenderNode *self)
   g_type_free_instance ((GTypeInstance *) self);
 }
 
-static void
-gsk_render_node_real_draw (GskRenderNode *node,
-                           cairo_t       *cr)
-{
-}
-
 static gboolean
 gsk_render_node_real_can_diff (const GskRenderNode *node1,
                                const GskRenderNode *node2)
@@ -159,20 +161,28 @@ gsk_render_node_real_diff (GskRenderNode  *node1,
   gsk_render_node_diff_impossible (node1, node2, data);
 }
 
+static gboolean
+gsk_render_node_real_get_opaque_rect (GskRenderNode   *node,
+                                      graphene_rect_t *out_opaque)
+{
+  return FALSE;
+}
+
 static void
 gsk_render_node_class_init (GskRenderNodeClass *klass)
 {
   klass->node_type = GSK_NOT_A_RENDER_NODE;
   klass->finalize = gsk_render_node_finalize;
-  klass->draw = gsk_render_node_real_draw;
   klass->can_diff = gsk_render_node_real_can_diff;
   klass->diff = gsk_render_node_real_diff;
+  klass->get_opaque_rect = gsk_render_node_real_get_opaque_rect;
 }
 
 static void
 gsk_render_node_init (GskRenderNode *self)
 {
   g_atomic_ref_count_init (&self->ref_count);
+  self->preferred_depth = GDK_N_DEPTHS; /* illegal value */
 }
 
 GType
@@ -262,7 +272,7 @@ gsk_render_node_type_register_static (const char     *node_name,
 
 /*< private >
  * gsk_render_node_alloc:
- * @node_type: the `GskRenderNode`Type to instantiate
+ * @node_type: the `GskRenderNode` type to instantiate
  *
  * Instantiates a new `GskRenderNode` for the given @node_type.
  *
@@ -280,11 +290,11 @@ gsk_render_node_alloc (GskRenderNodeType node_type)
 
 /**
  * gsk_render_node_ref:
- * @node: a `GskRenderNode`
+ * @node: a render node
  *
  * Acquires a reference on the given `GskRenderNode`.
  *
- * Returns: (transfer full): the `GskRenderNode` with an additional reference
+ * Returns: (transfer full): the render node with an additional reference
  */
 GskRenderNode *
 (gsk_render_node_ref) (GskRenderNode *node)
@@ -304,7 +314,7 @@ _gsk_render_node_unref (GskRenderNode *node)
 
 /**
  * gsk_render_node_unref:
- * @node: (transfer full): a `GskRenderNode`
+ * @node: (transfer full): a render node
  *
  * Releases a reference on the given `GskRenderNode`.
  *
@@ -322,30 +332,23 @@ void
 
 /**
  * gsk_render_node_get_node_type:
- * @node: a `GskRenderNode`
+ * @node: a render node
  *
- * Returns the type of the @node.
+ * Returns the type of the render node.
  *
- * Returns: the type of the `GskRenderNode`
+ * Returns: the type of @node
  */
 GskRenderNodeType
-gsk_render_node_get_node_type (const GskRenderNode *node)
+(gsk_render_node_get_node_type) (const GskRenderNode *node)
 {
   g_return_val_if_fail (GSK_IS_RENDER_NODE (node), GSK_NOT_A_RENDER_NODE);
 
   return GSK_RENDER_NODE_GET_CLASS (node)->node_type;
 }
 
-G_GNUC_PURE static inline
-GskRenderNodeType
-_gsk_render_node_get_node_type (const GskRenderNode *node)
-{
-  return GSK_RENDER_NODE_GET_CLASS (node)->node_type;
-}
-
 /**
  * gsk_render_node_get_bounds:
- * @node: a `GskRenderNode`
+ * @node: a render node
  * @bounds: (out caller-allocates): return location for the boundaries
  *
  * Retrieves the boundaries of the @node.
@@ -362,31 +365,17 @@ gsk_render_node_get_bounds (GskRenderNode   *node,
   graphene_rect_init_from_rect (bounds, &node->bounds);
 }
 
-/**
- * gsk_render_node_draw:
- * @node: a `GskRenderNode`
- * @cr: cairo context to draw to
- *
- * Draw the contents of @node to the given cairo context.
- *
- * Typically, you'll use this function to implement fallback rendering
- * of `GskRenderNode`s on an intermediate Cairo context, instead of using
- * the drawing context associated to a [class@Gdk.Surface]'s rendering buffer.
- *
- * For advanced nodes that cannot be supported using Cairo, in particular
- * for nodes doing 3D operations, this function may fail.
- **/
 void
-gsk_render_node_draw (GskRenderNode *node,
-                      cairo_t       *cr)
+gsk_render_node_draw_ccs (GskRenderNode *node,
+                          cairo_t       *cr,
+                          GdkColorState *ccs)
 {
-  g_return_if_fail (GSK_IS_RENDER_NODE (node));
-  g_return_if_fail (cr != NULL);
-  g_return_if_fail (cairo_status (cr) == CAIRO_STATUS_SUCCESS);
+  /* Check that the calling function did pass a correct color state */
+  g_assert (ccs == gdk_color_state_get_rendering_color_state (ccs));
 
   cairo_save (cr);
 
-  GSK_RENDER_NODE_GET_CLASS (node)->draw (node, cr);
+  GSK_RENDER_NODE_GET_CLASS (node)->draw (node, cr, ccs);
 
   if (GSK_DEBUG_CHECK (GEOMETRY))
     {
@@ -408,18 +397,72 @@ gsk_render_node_draw (GskRenderNode *node,
     }
 }
 
-/*
- * gsk_render_node_draw_fallback:
- * @node: a `GskRenderNode`
+void
+gsk_render_node_draw_with_color_state (GskRenderNode *node,
+                                       cairo_t       *cr,
+                                       GdkColorState *color_state)
+{
+  GdkColorState *ccs;
+
+  ccs = gdk_color_state_get_rendering_color_state (color_state);
+
+  if (gdk_color_state_equal (color_state, ccs))
+    {
+      gsk_render_node_draw_ccs (node, cr, ccs);
+    }
+  else
+    {
+      cairo_save (cr);
+      gdk_cairo_rect (cr, &node->bounds);
+      cairo_clip (cr);
+      cairo_push_group (cr);
+
+      gsk_render_node_draw_ccs (node, cr, ccs);
+      gdk_cairo_surface_convert_color_state (cairo_get_group_target (cr),
+                                             ccs,
+                                             color_state);
+      cairo_pop_group_to_source (cr);
+      cairo_paint (cr);
+      cairo_restore (cr);
+    }
+}
+
+/**
+ * gsk_render_node_draw:
+ * @node: a render node
  * @cr: cairo context to draw to
  *
- * Like gsk_render_node_draw(), but will overlay an error pattern if
- * GSK_DEBUG=cairo is enabled.
+ * Draws the contents of a render node on a cairo context.
  *
- * This has 2 purposes:
- * 1. It allows detecting fallbacks in GPU renderers.
- * 2. Application code can use it to detect where it is using Cairo
- *    drawing.
+ * Typically, you'll use this function to implement fallback rendering
+ * of render nodes on an intermediate Cairo context, instead of using
+ * the drawing context associated to a [class@Gdk.Surface]'s rendering buffer.
+ *
+ * For advanced nodes that cannot be supported using Cairo, in particular
+ * for nodes doing 3D operations, this function may fail.
+ **/
+void
+gsk_render_node_draw (GskRenderNode *node,
+                      cairo_t       *cr)
+{
+  g_return_if_fail (GSK_IS_RENDER_NODE (node));
+  g_return_if_fail (cr != NULL);
+  g_return_if_fail (cairo_status (cr) == CAIRO_STATUS_SUCCESS);
+
+  gsk_render_node_draw_with_color_state (node, cr, GDK_COLOR_STATE_SRGB);
+}
+
+/*
+ * gsk_render_node_draw_fallback:
+ * @node: a render node
+ * @cr: cairo context to draw to
+ *
+ * Like [method@Gsk.RenderNode.draw], but will overlay an error pattern
+ * if `GSK_DEBUG=cairo` is enabled.
+ *
+ * This has two purposes:
+ * 1. It allows detecting fallbacks in GPU renderers
+ * 2. Application code can use it to detect where it is using Cairo drawing
  *
  * So use this function whenever either of those cases should be detected.
  */
@@ -462,18 +505,18 @@ gsk_render_node_draw_fallback (GskRenderNode *node,
 
 /*
  * gsk_render_node_can_diff:
- * @node1: a `GskRenderNode`
- * @node2: the `GskRenderNode` to compare with
+ * @node1: a render node
+ * @node2: the render onde to compare with
  *
  * Checks if two render nodes can be expected to be compared via
- * gsk_render_node_diff().
+ * [method@Gsk.RenderNode.diff].
  *
  * The node diffing algorithm uses this function to match up similar
  * nodes to compare when trying to minimize the resulting region.
  *
- * Nodes of different type always return %FALSE here.
+ * Nodes of different type always return false here.
  *
- * Returns: %TRUE if @node1 and @node2 can be expected to be compared
+ * Returns: true if @node1 and @node2 can be expected to be compared
  **/
 gboolean
 gsk_render_node_can_diff (const GskRenderNode *node1,
@@ -482,11 +525,11 @@ gsk_render_node_can_diff (const GskRenderNode *node1,
   if (node1 == node2)
     return TRUE;
 
-  if (_gsk_render_node_get_node_type (node1) == _gsk_render_node_get_node_type (node2))
+  if (gsk_render_node_get_node_type (node1) == gsk_render_node_get_node_type (node2))
     return GSK_RENDER_NODE_GET_CLASS (node1)->can_diff (node1, node2);
 
-  if (_gsk_render_node_get_node_type (node1) == GSK_CONTAINER_NODE ||
-      _gsk_render_node_get_node_type (node2) == GSK_CONTAINER_NODE)
+  if (gsk_render_node_get_node_type (node1) == GSK_CONTAINER_NODE ||
+      gsk_render_node_get_node_type (node2) == GSK_CONTAINER_NODE)
     return TRUE;
 
   return FALSE;
@@ -517,11 +560,11 @@ gsk_render_node_diff_impossible (GskRenderNode  *node1,
 
 /**
  * gsk_render_node_diff:
- * @node1: a `GskRenderNode`
- * @node2: the `GskRenderNode` to compare with
- * @data: the diff data to use
+ * @node1: a render node
+ * @node2: the render node to compare with
+ * @data: diff data to use
  *
- * Compares @node1 and @node2 trying to compute the minimal region of changes.
+ * Compares two nodes, trying to compute the minimal region of changes.
  *
  * In the worst case, this is the union of the bounds of @node1 and @node2.
  *
@@ -541,15 +584,15 @@ gsk_render_node_diff (GskRenderNode  *node1,
   if (node1 == node2)
     return;
 
-  if (_gsk_render_node_get_node_type (node1) == _gsk_render_node_get_node_type (node2))
+  if (gsk_render_node_get_node_type (node1) == gsk_render_node_get_node_type (node2))
     {
       GSK_RENDER_NODE_GET_CLASS (node1)->diff (node1, node2, data);
     }
-  else if (_gsk_render_node_get_node_type (node1) == GSK_CONTAINER_NODE)
+  else if (gsk_render_node_get_node_type (node1) == GSK_CONTAINER_NODE)
     {
       gsk_container_node_diff_with (node1, node2, data);
     }
-  else if (_gsk_render_node_get_node_type (node2) == GSK_CONTAINER_NODE)
+  else if (gsk_render_node_get_node_type (node2) == GSK_CONTAINER_NODE)
     {
       gsk_container_node_diff_with (node2, node1, data);
     }
@@ -559,11 +602,46 @@ gsk_render_node_diff (GskRenderNode  *node1,
     }
 }
 
+/**
+ * gsk_render_node_get_opaque_rect:
+ * @self: a render node
+ * @out_opaque: (out): return location for the opaque rect
+ *
+ * Gets an opaque rectangle inside the node that GTK can determine to
+ * be fully opaque.
+ *
+ * There is no guarantee that this is indeed the largest opaque rectangle or
+ * that regions outside the rectangle are not opaque. This function is a best
+ * effort with that goal.
+ *
+ * The rectangle will be fully contained in the bounds of the node.
+ *
+ * Returns: true if part or all of the rendernode is opaque, false if no
+ *   opaque region could be found.
+ *
+ * Since: 4.16
+ **/
+gboolean
+gsk_render_node_get_opaque_rect (GskRenderNode   *self,
+                                 graphene_rect_t *out_opaque)
+{
+  g_return_val_if_fail (GSK_IS_RENDER_NODE (self), FALSE);
+  g_return_val_if_fail (out_opaque != NULL, FALSE);
+
+  if (self->fully_opaque)
+    {
+      *out_opaque = self->bounds;
+      return TRUE;
+    }
+
+  return GSK_RENDER_NODE_GET_CLASS (self)->get_opaque_rect (self, out_opaque);
+}
+
 /**
  * gsk_render_node_write_to_file:
- * @node: a `GskRenderNode`
- * @filename: (type filename): the file to save it to.
- * @error: Return location for a potential error
+ * @node: a render node
+ * @filename: (type filename): the file to save it to
+ * @error: return location for an error
  *
  * This function is equivalent to calling [method@Gsk.RenderNode.serialize]
  * followed by [func@GLib.file_set_contents].
@@ -573,7 +651,7 @@ gsk_render_node_diff (GskRenderNode  *node1,
  * It is mostly intended for use inside a debugger to quickly dump a render
  * node to a file for later inspection.
  *
- * Returns: %TRUE if saving was successful
+ * Returns: true if saving was successful
  **/
 gboolean
 gsk_render_node_write_to_file (GskRenderNode *node,
@@ -600,14 +678,14 @@ gsk_render_node_write_to_file (GskRenderNode *node,
 /**
  * gsk_render_node_deserialize:
  * @bytes: the bytes containing the data
- * @error_func: (nullable) (scope call): Callback on parsing errors
- * @user_data: (closure error_func): user_data for @error_func
+ * @error_func: (nullable) (scope call) (closure user_data): callback on parsing errors
+ * @user_data: user_data for @error_func
  *
  * Loads data previously created via [method@Gsk.RenderNode.serialize].
  *
  * For a discussion of the supported format, see that function.
  *
- * Returns: (nullable) (transfer full): a new `GskRenderNode`
+ * Returns: (nullable) (transfer full): a new render node
  */
 GskRenderNode *
 gsk_render_node_deserialize (GBytes            *bytes,
@@ -624,11 +702,12 @@ gsk_render_node_deserialize (GBytes            *bytes,
 /**
  * gsk_value_set_render_node:
  * @value: a [struct@GObject.Value] initialized with type `GSK_TYPE_RENDER_NODE`
- * @node: a `GskRenderNode`
+ * @node: a render node
  *
- * Stores the given `GskRenderNode` inside `value`.
+ * Stores the given render node inside a `GValue`.
  *
- * The [struct@GObject.Value] will acquire a reference to the `node`.
+ * The [struct@GObject.Value] will acquire a reference
+ * to the render node.
  *
  * Since: 4.6
  */
@@ -660,11 +739,12 @@ gsk_value_set_render_node (GValue        *value,
 /**
  * gsk_value_take_render_node:
  * @value: a [struct@GObject.Value] initialized with type `GSK_TYPE_RENDER_NODE`
- * @node: (transfer full) (nullable): a `GskRenderNode`
+ * @node: (transfer full) (nullable): a render node
  *
- * Stores the given `GskRenderNode` inside `value`.
+ * Stores the given render node inside a `GValue`.
  *
- * This function transfers the ownership of the `node` to the `GValue`.
+ * This function transfers the ownership of the
+ * render node to the `GValue`.
  *
  * Since: 4.6
  */
@@ -697,9 +777,9 @@ gsk_value_take_render_node (GValue        *value,
  * gsk_value_get_render_node:
  * @value: a `GValue` initialized with type `GSK_TYPE_RENDER_NODE`
  *
- * Retrieves the `GskRenderNode` stored inside the given `value`.
+ * Retrieves the render node stored inside a `GValue`.
  *
- * Returns: (transfer none) (nullable): a `GskRenderNode`
+ * Returns: (transfer none) (nullable): the render node
  *
  * Since: 4.6
  */
@@ -715,10 +795,10 @@ gsk_value_get_render_node (const GValue *value)
  * gsk_value_dup_render_node:
  * @value: a [struct@GObject.Value] initialized with type `GSK_TYPE_RENDER_NODE`
  *
- * Retrieves the `GskRenderNode` stored inside the given `value`, and acquires
- * a reference to it.
+ * Retrieves the render node stored inside a `GValue`,
+ * and acquires a reference to it.
  *
- * Returns: (transfer full) (nullable): a `GskRenderNode`
+ * Returns: (transfer full) (nullable): the render node
  *
  * Since: 4.6
  */
@@ -739,6 +819,12 @@ gsk_render_node_get_preferred_depth (const GskRenderNode *node)
   return node->preferred_depth;
 }
 
+gboolean
+gsk_render_node_is_hdr (const GskRenderNode *node)
+{
+  return node->is_hdr;
+}
+
 /* Whether we need an offscreen to handle opacity correctly for this node.
  * We don't if there is only one drawing node inside (could be child
  * node, or grandchild, or...).
