@@ -32,9 +32,9 @@
 /**
  * GtkAdjustment:
  *
- * `GtkAdjustment` is a model for a numeric value.
+ * A model for a numeric value.
  *
- * The `GtkAdjustment has an associated lower and upper bound.
+ * The `GtkAdjustment` has an associated lower and upper bound.
  * It also contains step and page increments, and a page size.
  *
  * Adjustments are used within several GTK widgets, including
@@ -131,7 +131,7 @@ gtk_adjustment_class_init (GtkAdjustmentClass *class)
   class->value_changed = NULL;
 
   /**
-   * GtkAdjustment:value: (attributes org.gtk.Property.get=gtk_adjustment_get_value org.gtk.Property.set=gtk_adjustment_set_value)
+   * GtkAdjustment:value:
    *
    * The value of the adjustment.
    */
@@ -142,7 +142,7 @@ gtk_adjustment_class_init (GtkAdjustmentClass *class)
                            GTK_PARAM_READWRITE);
 
   /**
-   * GtkAdjustment:lower: (attributes org.gtk.Property.get=gtk_adjustment_get_lower org.gtk.Property.set=gtk_adjustment_set_lower)
+   * GtkAdjustment:lower:
    *
    * The minimum value of the adjustment.
    */
@@ -153,7 +153,7 @@ gtk_adjustment_class_init (GtkAdjustmentClass *class)
                            GTK_PARAM_READWRITE);
 
   /**
-   * GtkAdjustment:upper: (attributes org.gtk.Property.get=gtk_adjustment_get_upper org.gtk.Property.set=gtk_adjustment_set_upper)
+   * GtkAdjustment:upper:
    *
    * The maximum value of the adjustment.
    *
@@ -167,7 +167,7 @@ gtk_adjustment_class_init (GtkAdjustmentClass *class)
                            GTK_PARAM_READWRITE);
 
   /**
-   * GtkAdjustment:step-increment: (attributes org.gtk.Property.get=gtk_adjustment_get_step_increment org.gtk.Property.set=gtk_adjustment_set_step_increment)
+   * GtkAdjustment:step-increment:
    *
    * The step increment of the adjustment.
    */
@@ -178,7 +178,7 @@ gtk_adjustment_class_init (GtkAdjustmentClass *class)
                            GTK_PARAM_READWRITE);
 
   /**
-   * GtkAdjustment:page-increment: (attributes org.gtk.Property.get=gtk_adjustment_get_page_increment org.gtk.Property.set=gtk_adjustment_set_page_increment)
+   * GtkAdjustment:page-increment:
    *
    * The page increment of the adjustment.
    */
@@ -189,7 +189,7 @@ gtk_adjustment_class_init (GtkAdjustmentClass *class)
                            GTK_PARAM_READWRITE);
 
   /**
-   * GtkAdjustment:page-size: (attributes org.gtk.Property.get=gtk_adjustment_get_page_size org.gtk.Property.set=gtk_adjustment_set_page_size)
+   * GtkAdjustment:page-size:
    *
    * The page size of the adjustment.
    *
@@ -207,7 +207,7 @@ gtk_adjustment_class_init (GtkAdjustmentClass *class)
 
   /**
    * GtkAdjustment::changed:
-   * @adjustment: the object which received the signal
+   * @adjustment: the adjustment which emitted the signal
    *
    * Emitted when one or more of the `GtkAdjustment` properties have been
    * changed.
@@ -226,7 +226,7 @@ gtk_adjustment_class_init (GtkAdjustmentClass *class)
 
   /**
    * GtkAdjustment::value-changed:
-   * @adjustment: the object which received the signal
+   * @adjustment: the adjustment which emitted the signal
    *
    * Emitted when the value has been changed.
    */
@@ -352,19 +352,23 @@ gtk_adjustment_dispatch_properties_changed (GObject     *object,
     }
 }
 
+double
+gtk_adjustment_get_bounded_upper (GtkAdjustment *self)
+{
+  GtkAdjustmentPrivate *priv = gtk_adjustment_get_instance_private (self);
+
+  /* The real upper value is upper - page_size and not below the lower value. */
+  return MAX (priv->lower, priv->upper - priv->page_size);
+}
+
 static double
 gtk_adjustment_sanitize_value (GtkAdjustment *self,
                                double         value)
 {
   GtkAdjustmentPrivate *priv = gtk_adjustment_get_instance_private (self);
+  gdouble upper = gtk_adjustment_get_bounded_upper (self);
 
-  /* don't use CLAMP() so we don't end up below lower if upper - page_size
-   * is smaller than lower
-   */
-  value = MIN (value, priv->upper - priv->page_size);
-  value = MAX (value, priv->lower);
-
-  return value;
+  return CLAMP (value, priv->lower, upper);
 }
 
 /**
@@ -399,12 +403,12 @@ gtk_adjustment_new (double value,
 }
 
 /**
- * gtk_adjustment_get_value: (attributes org.gtk.Method.get_property=value)
- * @adjustment: a `GtkAdjustment`
+ * gtk_adjustment_get_value:
+ * @adjustment: an adjustment
  *
  * Gets the current value of the adjustment.
  *
- * Returns: The current value of the adjustment
+ * Returns: the current value
  */
 double
 gtk_adjustment_get_value (GtkAdjustment *adjustment)
@@ -535,8 +539,8 @@ gtk_adjustment_set_value_internal (GtkAdjustment *adjustment,
 }
 
 /**
- * gtk_adjustment_set_value: (attributes org.gtk.Method.set_property=value)
- * @adjustment: a `GtkAdjustment`
+ * gtk_adjustment_set_value:
+ * @adjustment: an adjustment
  * @value: the new value
  *
  * Sets the `GtkAdjustment` value.
@@ -568,12 +572,12 @@ gtk_adjustment_animate_to_value (GtkAdjustment *adjustment,
 }
 
 /**
- * gtk_adjustment_get_lower: (attributes org.gtk.Method.get_property=lower)
- * @adjustment: a `GtkAdjustment`
+ * gtk_adjustment_get_lower:
+ * @adjustment: an adjustment
  *
  * Retrieves the minimum value of the adjustment.
  *
- * Returns: The current minimum value of the adjustment
+ * Returns: the minimum value
  **/
 double
 gtk_adjustment_get_lower (GtkAdjustment *adjustment)
@@ -586,8 +590,8 @@ gtk_adjustment_get_lower (GtkAdjustment *adjustment)
 }
 
 /**
- * gtk_adjustment_set_lower: (attributes org.gtk.Method.set_property=lower)
- * @adjustment: a `GtkAdjustment`
+ * gtk_adjustment_set_lower:
+ * @adjustment: an adjustment
  * @lower: the new minimum value
  *
  * Sets the minimum value of the adjustment.
@@ -620,12 +624,12 @@ gtk_adjustment_set_lower (GtkAdjustment *adjustment,
 }
 
 /**
- * gtk_adjustment_get_upper: (attributes org.gtk.Method.get_property=upper)
- * @adjustment: a `GtkAdjustment`
+ * gtk_adjustment_get_upper:
+ * @adjustment: an adjustment
  *
  * Retrieves the maximum value of the adjustment.
  *
- * Returns: The current maximum value of the adjustment
+ * Returns: the maximum value
  */
 double
 gtk_adjustment_get_upper (GtkAdjustment *adjustment)
@@ -638,8 +642,8 @@ gtk_adjustment_get_upper (GtkAdjustment *adjustment)
 }
 
 /**
- * gtk_adjustment_set_upper: (attributes org.gtk.Method.set_property=upper)
- * @adjustment: a `GtkAdjustment`
+ * gtk_adjustment_set_upper:
+ * @adjustment: an adjustment
  * @upper: the new maximum value
  *
  * Sets the maximum value of the adjustment.
@@ -667,12 +671,12 @@ gtk_adjustment_set_upper (GtkAdjustment *adjustment,
 }
 
 /**
- * gtk_adjustment_get_step_increment: (attributes org.gtk.Method.get_property=step-increment)
- * @adjustment: a `GtkAdjustment`
+ * gtk_adjustment_get_step_increment:
+ * @adjustment: an adjustment
  *
  * Retrieves the step increment of the adjustment.
  *
- * Returns: The current step increment of the adjustment.
+ * Returns: the step increment
  */
 double
 gtk_adjustment_get_step_increment (GtkAdjustment *adjustment)
@@ -685,8 +689,8 @@ gtk_adjustment_get_step_increment (GtkAdjustment *adjustment)
 }
 
 /**
- * gtk_adjustment_set_step_increment: (attributes org.gtk.Method.set_property=step-increment)
- * @adjustment: a `GtkAdjustment`
+ * gtk_adjustment_set_step_increment:
+ * @adjustment: an adjustment
  * @step_increment: the new step increment
  *
  * Sets the step increment of the adjustment.
@@ -711,12 +715,12 @@ gtk_adjustment_set_step_increment (GtkAdjustment *adjustment,
 }
 
 /**
- * gtk_adjustment_get_page_increment: (attributes org.gtk.Method.get_property=page-increment)
- * @adjustment: a `GtkAdjustment`
+ * gtk_adjustment_get_page_increment:
+ * @adjustment: an adjustment
  *
  * Retrieves the page increment of the adjustment.
  *
- * Returns: The current page increment of the adjustment
+ * Returns: the page increment
  **/
 double
 gtk_adjustment_get_page_increment (GtkAdjustment *adjustment)
@@ -729,8 +733,8 @@ gtk_adjustment_get_page_increment (GtkAdjustment *adjustment)
 }
 
 /**
- * gtk_adjustment_set_page_increment: (attributes org.gtk.Method.set_property=page-increment)
- * @adjustment: a `GtkAdjustment`
+ * gtk_adjustment_set_page_increment:
+ * @adjustment: an adjustment
  * @page_increment: the new page increment
  *
  * Sets the page increment of the adjustment.
@@ -755,12 +759,12 @@ gtk_adjustment_set_page_increment (GtkAdjustment *adjustment,
 }
 
 /**
- * gtk_adjustment_get_page_size: (attributes org.gtk.Method.get_property=page-size)
- * @adjustment: a `GtkAdjustment`
+ * gtk_adjustment_get_page_size:
+ * @adjustment: an adjustment
  *
  * Retrieves the page size of the adjustment.
  *
- * Returns: The current page size of the adjustment
+ * Returns: the page size
  **/
 double
 gtk_adjustment_get_page_size (GtkAdjustment *adjustment)
@@ -773,8 +777,8 @@ gtk_adjustment_get_page_size (GtkAdjustment *adjustment)
 }
 
 /**
- * gtk_adjustment_set_page_size: (attributes org.gtk.Method.set_property=page-size)
- * @adjustment: a `GtkAdjustment`
+ * gtk_adjustment_set_page_size:
+ * @adjustment: an adjustment
  * @page_size: the new page size
  *
  * Sets the page size of the adjustment.
@@ -800,7 +804,7 @@ gtk_adjustment_set_page_size (GtkAdjustment *adjustment,
 
 /**
  * gtk_adjustment_configure:
- * @adjustment: a `GtkAdjustment`
+ * @adjustment: an adjustment
  * @value: the new value
  * @lower: the new minimum value
  * @upper: the new maximum value
@@ -858,12 +862,12 @@ gtk_adjustment_configure (GtkAdjustment *adjustment,
 
 /**
  * gtk_adjustment_clamp_page:
- * @adjustment: a `GtkAdjustment`
+ * @adjustment: an adjustment
  * @lower: the lower value
  * @upper: the upper value
  *
- * Updates the value property to ensure that the range
- * between @lower and @upper is in the current page.
+ * Updates the value of the adjustment to ensure that the
+ * given range is contained in the current page.
  *
  * The current page goes from `value` to `value` + `page-size`.
  * If the range is larger than the page size, then only the
@@ -904,11 +908,11 @@ gtk_adjustment_clamp_page (GtkAdjustment *adjustment,
 
 /**
  * gtk_adjustment_get_minimum_increment:
- * @adjustment: a `GtkAdjustment`
+ * @adjustment: an adjustment
  *
  * Gets the smaller of step increment and page increment.
  *
- * Returns: the minimum increment of @adjustment
+ * Returns: the minimum increment
  */
 double
 gtk_adjustment_get_minimum_increment (GtkAdjustment *adjustment)
