@@ -278,6 +278,41 @@ gtk_application_set_window_icon (GtkApplication *application)
   gtk_window_set_default_icon_name (appid);
 }
 
+#if defined(GDK_WINDOWING_WAYLAND) || defined(GDK_WINDOWING_X11)
+static void
+gtk_application_identify_to_portal (GtkApplication *application)
+{
+  GVariantBuilder builder;
+  GApplication *g_application = G_APPLICATION (application);
+  GDBusConnection *session_bus;
+  const char *application_id;
+
+  session_bus = g_application_get_dbus_connection (g_application);
+  if (!session_bus)
+    return;
+
+  application_id = g_application_get_application_id (g_application);
+  if (!application_id)
+    return;
+
+  g_variant_builder_init (&builder, G_VARIANT_TYPE_VARDICT);
+
+  /* Intentionally ignore errors */
+  g_dbus_connection_call (session_bus,
+                          PORTAL_BUS_NAME,
+                          PORTAL_OBJECT_PATH,
+                          "org.freedesktop.host.portal.Registry",
+                          "Register",
+                          g_variant_new ("(sa{sv})",
+                                         application_id,
+                                         &builder),
+                          NULL,
+                          G_DBUS_CALL_FLAGS_NO_AUTO_START,
+                          -1,
+                          NULL, NULL, NULL);
+}
+#endif
+
 static void
 gtk_application_startup (GApplication *g_application)
 {
@@ -292,6 +327,11 @@ gtk_application_startup (GApplication *g_application)
 
   gtk_action_muxer_insert (priv->muxer, "app", G_ACTION_GROUP (application));
 
+#if defined(GDK_WINDOWING_WAYLAND) || defined(GDK_WINDOWING_X11)
+  if (!gdk_running_in_sandbox ())
+    gtk_application_identify_to_portal (application);
+#endif
+
   before2 = GDK_PROFILER_CURRENT_TIME;
   gtk_init ();
   gdk_profiler_end_mark (before2, "gtk_init", NULL);
@@ -980,7 +1020,7 @@ gtk_application_get_menubar (GtkApplication *application)
  * If a window is given, the session manager may point the user to
  * this window to find out more about why the action is inhibited.
  *
- * The cookie tha tis returned by this function  should be used as an
+ * The cookie that is returned by this function  should be used as an
  * argument to [method@Gtk.Application.uninhibit] in order to remove
  * the request.
  *
